@@ -15,10 +15,10 @@
   ((layeredp :initarg :layered
              :initform nil
              :reader slot-definition-layeredp)
-   (layered-readers :initarg layered-readers
+   (layered-readers :initarg :layered-readers
                     :initform ()
                     :reader slot-definition-layered-readers)
-   (layered-writers :initarg layered-writers
+   (layered-writers :initarg :layered-writers
                     :initform ()
                     :reader slot-definition-layered-writers)
    (layered-accessor-methods :initform ()
@@ -90,32 +90,33 @@
   (flet ((writer () (call-next-method)))
     (slot-makunbound-using-layer class object slot #'writer)))
 
-(defmethod initialize-instance :around
-  ((slot layered-direct-slot-definition) &rest initargs
-   &key layered-reader layered-writer layered-accessor)
-  (declare (dynamic-extent initargs)
-           (ignore layered-reader layered-writer layered-accessor))
-  (loop for (key value) on initargs by #'cddr
-        if (eq key :layered-reader)
-        collect value into layered-readers
-        else if (eq key :layered-writer)
-        collect value into layered-writers
-        else if (eq key :layered-accessor)
-        collect value into layered-readers
-        and collect `(setf ,value) into layered-writers
-        else nconc (list key value) into other-initargs
-        finally (apply #'call-next-method slot
-                       'layered-readers layered-readers
-                       'layered-writers layered-writers
-                       other-initargs)))
+(defgeneric process-layered-access-slot-specification (slot-spec)
+  (:method ((slot-spec symbol)) slot-spec)
+  (:method ((slot-spec cons))
+   (let ((plist (cdr slot-spec)))
+     (if (get-properties plist '(:layered-reader :layered-writer :layered-accessor))
+       (loop for (key value) on plist by #'cddr
+             if (eq key :layered-reader)
+             collect value into layered-readers
+             else if (eq key :layered-writer)
+             collect value into layered-writers
+             else if (eq key :layered-accessor)
+             collect value into layered-readers
+             and collect `(setf ,value) into layered-writers
+             else nconc (list key value) into other-initargs
+             finally (return (list* (car slot-spec)
+                                    :layered-readers layered-readers
+                                    :layered-writers layered-writers
+                                    other-initargs)))
+       slot-spec))))
 
-(defgeneric add-layered-accessors (class accessor-method-layer)
-  (:method ((class layered-access-class) accessor-method-layer)
-   (loop with layer = (find-layer-class accessor-method-layer)
-         with reader-specializers = (list class)
+(defgeneric add-layered-accessors (class)
+  (:method ((class layered-access-class))
+   (loop with reader-specializers = (list class)
          with writer-specializers = (list (find-class 't) class)
          for slot in (class-direct-slots class)
-         for slot-name = (slot-definition-name slot) do
+         for slot-name = (slot-definition-name slot)
+         for layer = (find-layer-class (slot-definition-layer slot)) do
          (loop for layered-reader in (slot-definition-layered-readers slot)
                for gf = (ensure-layered-function layered-reader :lambda-list '(object))
                for method = (ensure-layered-method
@@ -149,17 +150,17 @@
                   do (remove-method (car method) (cdr method))))))
 
 (defmethod initialize-instance :after
-  ((class layered-access-class) &key (in-layer '(t)))
-  (add-layered-accessors class (car in-layer)))
+  ((class layered-access-class) &key)
+  (add-layered-accessors class))
 
 (defmethod reinitialize-instance :around
   ((class layered-access-class)
-   &key (in-layer '(t)) (direct-slots () direct-slots-p))
+   &key (direct-slots () direct-slots-p))
   (declare (ignore direct-slots))
   (if direct-slots-p
-      (progn
-        (remove-layered-accessors class)
-        (call-next-method)
-        (add-layered-accessors class (car in-layer))
-        class)
+    (progn
+      (remove-layered-accessors class)
+      (call-next-method)
+      (add-layered-accessors class)
+      class)
     (call-next-method)))
