@@ -5,6 +5,7 @@
         &rest initargs
         &key (lambda-list () lambda-list-p)
         (argument-precedence-order (required-args lambda-list))
+        (generic-function-class 'layered-function)
         &allow-other-keys)
   (declare (dynamic-extent initargs))
   (unless lambda-list-p
@@ -12,6 +13,8 @@
   (let ((gf (let ((layer-arg (gensym "LAYER-ARG-")))
               (apply #'ensure-generic-function
                      (get-layered-function-definer-name name)
+		     #-allegro :generic-function-class
+                     #-allegro generic-function-class
                      :argument-precedence-order
                      `(,@argument-precedence-order ,layer-arg)
                      :lambda-list
@@ -25,8 +28,18 @@
                           (apply (the function ,gf)
                                  (layer-context-prototype *active-context*)
                                  rest))))
+    (bind-layered-function-names name)
     gf))
 
+(defun layered-function-definer (name)
+  (fdefinition (get-layered-function-definer-name name)))
+
+(defgeneric layered-function-argument-precedence-order (function)
+  (:method ((function layered-function)) (butlast (generic-function-argument-precedence-order function))))
+
+(defgeneric layered-function-lambda-list (function)
+  (:method ((function layered-function)) (rest (generic-function-lambda-list function))))
+
 (defun lfmakunbound (name)
   (fmakunbound (get-layered-function-definer-name name))
   (fmakunbound name))
@@ -39,7 +52,7 @@
         (method-class
          (generic-function-method-class
           (fdefinition (get-layered-function-definer-name layered-function-name))))
-        (in-layer (find-layer-class 't))
+        (in-layer 't)
         (qualifiers ())
         (lambda-list (cadr lambda-expression))
         (specializers (required-args lambda-list (constantly (find-class 't)))))
@@ -55,4 +68,24 @@
                      #-(or allegro clisp cmu ecl mcl openmcl) method-class
                      :qualifiers qualifiers
                      :lambda-list `(,layer-arg ,@lambda-list)
-                     :specializers (cons (prepare-layer in-layer) specializers)))))
+                     :specializers (cons (find-layer-class in-layer) specializers)))))
+
+(defgeneric layered-method-lambda-list (method)
+  (:method ((method layered-method)) (rest (method-lambda-list method))))
+
+(defgeneric layered-method-specializers (method)
+  (:method ((method layered-method)) (rest (method-specializers method))))
+
+(defgeneric layered-method-layer (method)
+  (:method ((method layered-method)) (find-layer (first (method-specializers method)))))
+
+(defmethod print-object ((object layered-method) stream)
+  (print-unreadable-object (object stream :type t :identity t)
+    (format stream "~A ~A ~S ~A"
+            (when (method-generic-function object)
+              (get-layered-function-caller-name
+               (generic-function-name
+                (method-generic-function object))))
+            (layer-name (layered-method-layer object))
+            (method-qualifiers object)
+            (mapcar #'class-name (layered-method-specializers object)))))
