@@ -16,7 +16,7 @@
    &key direct-superclasses)
   (declare (dynamic-extent initargs))
   (if (loop for superclass in direct-superclasses
-            thereis (subtypep superclass 'special-object))
+            thereis (ignore-errors (subtypep superclass 'special-object)))
     (call-next-method)
     (apply #'call-next-method class
            :direct-superclasses
@@ -30,7 +30,7 @@
   (declare (dynamic-extent initargs))
   (if direct-superclasses-p
     (if (loop for superclass in direct-superclasses
-              thereis (subtypep superclass 'special-object))
+              thereis (ignore-errors (subtypep superclass 'special-object)))
       (call-next-method)
       (apply #'call-next-method class
              :direct-superclasses
@@ -74,25 +74,25 @@
 
 (defun shift-slot (object slot-name)
   (with-symbol-access
-    (when (slot-boundp object slot-name)
-      (let ((slot-value (slot-value object slot-name)))
-        (unless (special-symbol-p slot-value)
-          (slot-makunbound object slot-name)
-          (without-symbol-access
-            (setf (slot-value object slot-name) slot-value)))))))
+    (let ((slot-value (slot-value object slot-name)))
+      (unless (special-symbol-p slot-value)
+        (slot-makunbound object slot-name)
+        (without-symbol-access
+          (setf (slot-value object slot-name) slot-value))))))
 
-#+(or lispworks (and mcl (not openmcl)))
 (defmethod shared-initialize ((object special-object) slot-names &rest all-keys)
   (declare (dynamic-extent all-keys))
   (let ((class-slots (class-slots (class-of object))))
-    (loop for slot in class-slots
-          for slot-initargs = (slot-definition-initargs slot)
-          when slot-initargs do
-          (multiple-value-bind
-              (indicator value)
-              (get-properties all-keys slot-initargs)
-            (when indicator
-              (setf (slot-value object (slot-definition-name slot)) value))))
+    (loop for slot in class-slots do
+          (when (and (typep slot 'special-effective-slot-definition)
+                     (not (eq (slot-definition-allocation slot) :class)))
+            (shift-slot object (slot-definition-name slot)))
+          (when-let (slot-initargs (slot-definition-initargs slot))
+            (multiple-value-bind
+                (indicator value)
+                (get-properties all-keys slot-initargs)
+              (when indicator
+                (setf (slot-value object (slot-definition-name slot)) value)))))
     (if (eq slot-names 't)
       (loop for slot in class-slots
             for slot-name = (slot-definition-name slot)
@@ -108,50 +108,35 @@
                 (setf (slot-value object slot-name) (funcall slot-initfunction)))))))
   object)
 
-(defmethod shared-initialize :after
-  ((object special-object) slot-names &rest all-keys)
-  "ensure that all initialized special slots are indeed bound to
-   a special slot; circumvents possible optimizations in the
-   initialization of standard-class objects"
-  (declare (ignore slot-names all-keys))
-  (loop for slot in (class-slots (class-of object))
-        when (and (typep slot 'special-effective-slot-definition)
-                  (not (eq (slot-definition-allocation slot) :class)))
-        do (shift-slot object (slot-definition-name slot))))
-
-(defvar *slot-unbound-p* nil)
-
-(defmethod slot-unbound :around ((class special-class) object slot-name)
+(defmethod slot-unbound ((class special-class) object slot-name)
   (declare (optimize (speed 3) (debug 0) (safety 0)
                      (compilation-speed 0)))
-  (if *slot-unbound-p*
-      (call-next-method)
+  (if *symbol-access*
     (let ((slot (find slot-name (the list (class-slots class))
                       :test #'eq
                       :key #'slot-definition-name)))
       (if (typep slot 'special-effective-slot-definition)
-          (with-symbol-access
-            (setf (slot-value-using-class class object slot)
-                  (make-special-symbol)))
-        (call-next-method)))))
+        (setf (slot-value-using-class class object slot)
+              (make-special-symbol))
+        (call-next-method)))
+    (call-next-method)))
 
 (defmethod slot-value-using-class
            ((class special-class) object (slot special-effective-slot-definition))
   (declare (optimize (speed 3) (debug 0) (safety 0)
                      (compilation-speed 0)))
-  (let ((slot-symbol (call-next-method)))
-    (declare (type symbol slot-symbol))
-    (cond (*symbol-access* slot-symbol)
-          ((boundp slot-symbol) (symbol-value slot-symbol))
-          (t (let ((*slot-unbound-p* t))
-               (slot-unbound class object (slot-definition-name slot)))))))
+  (if *symbol-access* (call-next-method)
+    (let ((slot-symbol (with-symbol-access (call-next-method))))
+      (declare (type symbol slot-symbol))
+      (if (boundp slot-symbol)
+        (symbol-value slot-symbol)
+        (slot-unbound class object (slot-definition-name slot))))))
 
 (defmethod (setf slot-value-using-class)
            (new-value (class special-class) object (slot special-effective-slot-definition))
   (declare (optimize (speed 3) (debug 0) (safety 0)
                      (compilation-speed 0)))
-  (if *symbol-access*
-      (call-next-method)
+  (if *symbol-access* (call-next-method)
     (let ((slot-symbol (with-symbol-access (slot-value-using-class class object slot))))
       (setf (symbol-value (the symbol slot-symbol)) new-value))))
 
@@ -159,8 +144,7 @@
            ((class special-class) object (slot special-effective-slot-definition))
   (declare (optimize (speed 3) (debug 0) (safety 0)
                      (compilation-speed 0)))
-  (if *symbol-access*
-      (call-next-method)
+  (if *symbol-access* (call-next-method)
     (let ((slot-symbol (with-symbol-access (slot-value-using-class class object slot))))
       (boundp (the symbol slot-symbol)))))
 
@@ -168,18 +152,26 @@
            ((class special-class) object (slot special-effective-slot-definition))
   (declare (optimize (speed 3) (debug 0) (safety 0)
                      (compilation-speed 0)))
-  (if *symbol-access*
-      (call-next-method)
+  (if *symbol-access* (call-next-method)
     (let ((slot-symbol (with-symbol-access (slot-value-using-class class object slot))))
       (makunbound (the symbol slot-symbol))
       object)))
 
+#+(or allegro lispworks)
+(defmethod make-instances-obsolete :after ((class special-class))
+  (mapc #'make-instances-obsolete (class-direct-subclasses class)))
+
 (defmethod reinitialize-instance :before
   ((class special-class) &key)
   (when (class-finalized-p class)
     (setf (slot-value class 'old-slot-definitions)
           (class-slots class))))
 
+#+cmu
+(defmethod reinitialize-instance :after
+  ((class special-class) &key)
+  (finalize-inheritance class))
+
 (defmethod finalize-inheritance :after
   ((class special-class))
   "ensure that special slots remain special after class redefinition
@@ -205,8 +197,10 @@
                              (make-instances-obsolete class))
                            t)))
         ()
-      #+(and allegro (not (version>= 7 0))) "The (non-)special slots in class ~S must remain (non-)special."
-      #-(and allegro (not (version>= 7 0))) "The special slots in class ~S must remain special."
+      #+(and allegro (not (version>= 7 0)))
+      "The (non-)special slots in class ~S must remain (non-)special."
+      #-(and allegro (not (version>= 7 0)))
+      "The special slots in class ~S must remain special."
       (class-name class))
     (slot-makunbound class 'old-slot-definitions))
     
